@@ -33,7 +33,6 @@ import uuid
 import warnings
 
 import debtcollector
-from positional import positional
 
 _request_store = threading.local()
 
@@ -179,7 +178,6 @@ class RequestContext(object):
 
     user_idt_format = u'{user} {tenant} {domain} {user_domain} {p_domain}'
 
-    @positional(enforcement=positional.WARN)
     def __init__(self,
                  auth_token=None,
                  user=None,
@@ -393,7 +391,7 @@ class RequestContext(object):
         """
         # Load a new context object from the environment variables set by
         # auth_token middleware. See:
-        # https://docs.openstack.org/developer/keystonemiddleware/api/keystonemiddleware.auth_token.html#what-auth-token-adds-to-the-request-for-use-by-the-openstack-service
+        # https://docs.openstack.org/keystonemiddleware/latest/api/keystonemiddleware.auth_token.html#what-auth-token-adds-to-the-request-for-use-by-the-openstack-service
 
         # add kwarg if not specified by user from a list of possible headers
         for k, v_list in _ENVIRON_HEADERS.items():
