@@ -43,6 +43,12 @@ class WarningsFixture(fixtures.Fixture):
         self.addCleanup(self._w.__exit__)
         warnings.simplefilter(self.action, self.category)
 
+    def __len__(self):
+        return len(self.log)
+
+    def __getitem__(self, item):
+        return self.log[item]
+
 
 class Object(object):
     pass
@@ -93,7 +99,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIsInstance(ctx, context.RequestContext)
         self.assertTrue(ctx.is_admin)
         self.assertFalse(ctx.show_deleted)
-        self.assertIsNone(ctx.tenant)
+        self.assertIsNone(ctx.project_id)
 
     def test_admin_context_show_deleted_flag_set(self):
         ctx = context.get_admin_context(show_deleted=True)
@@ -117,20 +123,22 @@ class ContextTest(test_base.BaseTestCase):
             "read_only": True,
             "show_deleted": True,
             "request_id": "request1",
+            "global_request_id": "req-uuid",
             "resource_uuid": "instance1",
             "extra_data": "foo"
         }
         ctx = context.RequestContext.from_dict(dct)
         self.assertEqual(dct['auth_token'], ctx.auth_token)
-        self.assertEqual(dct['user'], ctx.user)
-        self.assertEqual(dct['tenant'], ctx.tenant)
-        self.assertEqual(dct['domain'], ctx.domain)
-        self.assertEqual(dct['user_domain'], ctx.user_domain)
-        self.assertEqual(dct['project_domain'], ctx.project_domain)
+        self.assertEqual(dct['user'], ctx.user_id)
+        self.assertEqual(dct['tenant'], ctx.project_id)
+        self.assertEqual(dct['domain'], ctx.domain_id)
+        self.assertEqual(dct['user_domain'], ctx.user_domain_id)
+        self.assertEqual(dct['project_domain'], ctx.project_domain_id)
         self.assertTrue(ctx.is_admin)
         self.assertTrue(ctx.read_only)
         self.assertTrue(ctx.show_deleted)
         self.assertEqual(dct['request_id'], ctx.request_id)
+        self.assertEqual(dct['global_request_id'], ctx.global_request_id)
         self.assertEqual(dct['resource_uuid'], ctx.resource_uuid)
         self.assertEqual(dct['user_name'], ctx.user_name)
         self.assertEqual(dct['project_name'], ctx.project_name)
@@ -149,8 +157,8 @@ class ContextTest(test_base.BaseTestCase):
         }
         ctx = context.RequestContext.from_dict(dct)
         self.assertEqual("token1", ctx.auth_token)
-        self.assertEqual("user1", ctx.user)
-        self.assertIsNone(ctx.tenant)
+        self.assertEqual("user1", ctx.user_id)
+        self.assertIsNone(ctx.project_id)
         self.assertFalse(ctx.is_admin)
         self.assertTrue(ctx.read_only)
         self.assertRaises(KeyError, lambda: ctx.__dict__['color'])
@@ -194,32 +202,70 @@ class ContextTest(test_base.BaseTestCase):
         project_domain_id = generate_id(project_domain_name)
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
         request_id = uuid.uuid4().hex
+        global_request_id = uuid.uuid4().hex
+        service_token = uuid.uuid4().hex
+        service_user_id = uuid.uuid4().hex
+        service_user_name = uuid.uuid4().hex
+        service_user_domain_id = uuid.uuid4().hex
+        service_user_domain_name = uuid.uuid4().hex
+        service_project_id = uuid.uuid4().hex
+        service_project_name = uuid.uuid4().hex
+        service_project_domain_id = uuid.uuid4().hex
+        service_project_domain_name = uuid.uuid4().hex
+        service_roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
-        environ = {'HTTP_X_AUTH_TOKEN': auth_token,
-                   'HTTP_X_USER_ID': user_id,
-                   'HTTP_X_PROJECT_ID': project_id,
-                   'HTTP_X_USER_DOMAIN_ID': user_domain_id,
-                   'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
-                   'HTTP_X_ROLES': ','.join(roles),
-                   'HTTP_X_USER_NAME': user_name,
-                   'HTTP_X_PROJECT_NAME': project_name,
-                   'HTTP_X_USER_DOMAIN_NAME': user_domain_name,
-                   'HTTP_X_PROJECT_DOMAIN_NAME': project_domain_name,
-                   'openstack.request_id': request_id}
+        environ = {
+            'HTTP_X_AUTH_TOKEN': auth_token,
+            'HTTP_X_USER_ID': user_id,
+            'HTTP_X_PROJECT_ID': project_id,
+            'HTTP_X_USER_DOMAIN_ID': user_domain_id,
+            'HTTP_X_PROJECT_DOMAIN_ID': project_domain_id,
+            'HTTP_X_ROLES': ','.join(roles),
+            'HTTP_X_USER_NAME': user_name,
+            'HTTP_X_PROJECT_NAME': project_name,
+            'HTTP_X_USER_DOMAIN_NAME': user_domain_name,
+            'HTTP_X_PROJECT_DOMAIN_NAME': project_domain_name,
+            'HTTP_X_SERVICE_TOKEN': service_token,
+            'HTTP_X_SERVICE_USER_ID': service_user_id,
+            'HTTP_X_SERVICE_USER_NAME': service_user_name,
+            'HTTP_X_SERVICE_USER_DOMAIN_ID': service_user_domain_id,
+            'HTTP_X_SERVICE_USER_DOMAIN_NAME': service_user_domain_name,
+            'HTTP_X_SERVICE_PROJECT_ID': service_project_id,
+            'HTTP_X_SERVICE_PROJECT_NAME': service_project_name,
+            'HTTP_X_SERVICE_PROJECT_DOMAIN_ID': service_project_domain_id,
+            'HTTP_X_SERVICE_PROJECT_DOMAIN_NAME': service_project_domain_name,
+            'HTTP_X_SERVICE_ROLES': ','.join(service_roles),
+            'openstack.request_id': request_id,
+            'openstack.global_request_id': global_request_id,
+        }
 
         ctx = context.RequestContext.from_environ(environ)
 
         self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_id, ctx.user_id)
         self.assertEqual(user_name, ctx.user_name)
-        self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_id, ctx.project_id)
         self.assertEqual(project_name, ctx.project_name)
-        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_id, ctx.user_domain_id)
         self.assertEqual(user_domain_name, ctx.user_domain_name)
-        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_id, ctx.project_domain_id)
         self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(roles, ctx.roles)
         self.assertEqual(request_id, ctx.request_id)
+        self.assertEqual(global_request_id, ctx.global_request_id)
+        self.assertEqual(service_token, ctx.service_token)
+        self.assertEqual(service_user_id, ctx.service_user_id)
+        self.assertEqual(service_user_name, ctx.service_user_name)
+        self.assertEqual(service_user_domain_id, ctx.service_user_domain_id)
+        self.assertEqual(service_user_domain_name,
+                         ctx.service_user_domain_name)
+        self.assertEqual(service_project_id, ctx.service_project_id)
+        self.assertEqual(service_project_name, ctx.service_project_name)
+        self.assertEqual(service_project_domain_id,
+                         ctx.service_project_domain_id)
+        self.assertEqual(service_project_domain_name,
+                         ctx.service_project_domain_name)
+        self.assertEqual(service_roles, ctx.service_roles)
 
     def test_from_environ_no_roles(self):
         ctx = context.RequestContext.from_environ(environ={})
@@ -237,7 +283,7 @@ class ContextTest(test_base.BaseTestCase):
 
         environ = {'HTTP_X_TENANT_ID': value}
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(value, ctx.tenant)
+        self.assertEqual(value, ctx.project_id)
 
         environ = {'HTTP_X_STORAGE_TOKEN': value}
         ctx = context.RequestContext.from_environ(environ=environ)
@@ -274,11 +320,11 @@ class ContextTest(test_base.BaseTestCase):
                    'HTTP_X_PROJECT_ID': new}
 
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(new, ctx.tenant)
+        self.assertEqual(new, ctx.project_id)
 
         ctx = context.RequestContext.from_environ(environ=environ,
                                                   tenant=override)
-        self.assertEqual(override, ctx.tenant)
+        self.assertEqual(override, ctx.project_id)
 
         environ = {'HTTP_X_TENANT_NAME': old,
                    'HTTP_X_PROJECT_NAME': new}
@@ -287,9 +333,11 @@ class ContextTest(test_base.BaseTestCase):
         self.assertEqual(new, ctx.project_name)
 
     def test_from_environ_strip_roles(self):
-        environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n'}
+        environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n',
+                   'HTTP_X_SERVICE_ROLES': ' jkl\t,\nmno\n,pqr\n\n'}
         ctx = context.RequestContext.from_environ(environ=environ)
         self.assertEqual(['abc', 'def', 'ghi'], ctx.roles)
+        self.assertEqual(['jkl', 'mno', 'pqr'], ctx.service_roles)
 
     def test_environ_admin_project(self):
         environ = {}
@@ -339,6 +387,7 @@ class ContextTest(test_base.BaseTestCase):
         read_only = True
         show_deleted = True
         request_id = "id1"
+        global_request_id = "req-id1"
         resource_uuid = "uuid1"
 
         ctx = context.RequestContext(auth_token=auth_token,
@@ -356,17 +405,18 @@ class ContextTest(test_base.BaseTestCase):
                                      read_only=read_only,
                                      show_deleted=show_deleted,
                                      request_id=request_id,
+                                     global_request_id=global_request_id,
                                      resource_uuid=resource_uuid)
         self.assertEqual(auth_token, ctx.auth_token)
-        self.assertEqual(user_id, ctx.user)
+        self.assertEqual(user_id, ctx.user_id)
         self.assertEqual(user_name, ctx.user_name)
-        self.assertEqual(project_id, ctx.tenant)
+        self.assertEqual(project_id, ctx.project_id)
         self.assertEqual(project_name, ctx.project_name)
-        self.assertEqual(domain_id, ctx.domain)
+        self.assertEqual(domain_id, ctx.domain_id)
         self.assertEqual(domain_name, ctx.domain_name)
-        self.assertEqual(user_domain_id, ctx.user_domain)
+        self.assertEqual(user_domain_id, ctx.user_domain_id)
         self.assertEqual(user_domain_name, ctx.user_domain_name)
-        self.assertEqual(project_domain_id, ctx.project_domain)
+        self.assertEqual(project_domain_id, ctx.project_domain_id)
         self.assertEqual(project_domain_name, ctx.project_domain_name)
         self.assertEqual(is_admin, ctx.is_admin)
         self.assertEqual(read_only, ctx.read_only)
@@ -420,6 +470,7 @@ class ContextTest(test_base.BaseTestCase):
         self.assertIn('read_only', d)
         self.assertIn('show_deleted', d)
         self.assertIn('request_id', d)
+        self.assertIn('global_request_id', d)
         self.assertIn('resource_uuid', d)
         self.assertIn('user_identity', d)
         self.assertIn('roles', d)
@@ -449,26 +500,43 @@ class ContextTest(test_base.BaseTestCase):
         id2 = context.generate_request_id()
         self.assertNotEqual(id1, id2)
 
+    def test_no_global_id_by_default(self):
+        ctx = context.RequestContext()
+        self.assertIsNone(ctx.global_request_id)
+        d = ctx.to_dict()
+        self.assertIsNone(d['global_request_id'])
+
     def test_policy_dict(self):
         user = uuid.uuid4().hex
         user_domain = uuid.uuid4().hex
         tenant = uuid.uuid4().hex
         project_domain = uuid.uuid4().hex
         roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
+        service_user_id = uuid.uuid4().hex
+        service_project_id = uuid.uuid4().hex
+        service_roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
 
         # default is_admin_project is True
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
                                      tenant=tenant,
                                      project_domain=project_domain,
-                                     roles=roles)
+                                     roles=roles,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
 
         self.assertEqual({'user_id': user,
                           'user_domain_id': user_domain,
                           'project_id': tenant,
                           'project_domain_id': project_domain,
                           'roles': roles,
-                          'is_admin_project': True},
+                          'is_admin_project': True,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
                          ctx.to_policy_values())
 
         ctx = context.RequestContext(user=user,
@@ -476,23 +544,24 @@ class ContextTest(test_base.BaseTestCase):
                                      tenant=tenant,
                                      project_domain=project_domain,
                                      roles=roles,
-                                     is_admin_project=False)
+                                     is_admin_project=False,
+                                     service_user_id=service_user_id,
+                                     service_project_id=service_project_id,
+                                     service_roles=service_roles)
 
         self.assertEqual({'user_id': user,
                           'user_domain_id': user_domain,
                           'project_id': tenant,
                           'project_domain_id': project_domain,
                           'roles': roles,
-                          'is_admin_project': False},
+                          'is_admin_project': False,
+                          'service_user_id': service_user_id,
+                          'service_user_domain_id': None,
+                          'service_project_id': service_project_id,
+                          'service_project_domain_id': None,
+                          'service_roles': service_roles},
                          ctx.to_policy_values())
 
-    def test_positional_args(self):
-        context.RequestContext('abc', 'def')
-
-        self.assertEqual(1, len(self.warnings.log))
-        self.assertIn('__init__ takes at most 1 positional',
-                      str(self.warnings.log[0].message))
-
     def test_policy_deprecations(self):
         user = uuid.uuid4().hex
         user_domain = uuid.uuid4().hex
@@ -521,3 +590,40 @@ class ContextTest(test_base.BaseTestCase):
             self.assertIs(val, policy[key])
             self.assertEqual(1, len(w))
             self.assertIn(key, str(w[0].message))
+
+    def test_deprecated_args(self):
+        user = uuid.uuid4().hex
+        tenant = uuid.uuid4().hex
+        domain = uuid.uuid4().hex
+        user_domain = uuid.uuid4().hex
+        project_domain = uuid.uuid4().hex
+
+        ctx = context.RequestContext(user=user,
+                                     tenant=tenant,
+                                     domain=domain,
+                                     user_domain=user_domain,
+                                     project_domain=project_domain)
+
+        self.assertEqual(0, len(self.warnings))
+        self.assertEqual(user, ctx.user_id)
+        self.assertEqual(tenant, ctx.project_id)
+        self.assertEqual(domain, ctx.domain_id)
+        self.assertEqual(user_domain, ctx.user_domain_id)
+        self.assertEqual(project_domain, ctx.project_domain_id)
+
+        self.assertEqual(0, len(self.warnings))
+        self.assertEqual(user, ctx.user)
+        if context._log_deprecation_warnings:
+            self.assertEqual(1, len(self.warnings))
+        self.assertEqual(tenant, ctx.tenant)
+        if context._log_deprecation_warnings:
+            self.assertEqual(2, len(self.warnings))
+        self.assertEqual(domain, ctx.domain)
+        if context._log_deprecation_warnings:
+            self.assertEqual(3, len(self.warnings))
+        self.assertEqual(user_domain, ctx.user_domain)
+        if context._log_deprecation_warnings:
+            self.assertEqual(4, len(self.warnings))
+        self.assertEqual(project_domain, ctx.project_domain)
+        if context._log_deprecation_warnings:
+            self.assertEqual(5, len(self.warnings))
